function UpdateChecker(){
	var _self = this;
	this.downloader = new FeedDownloader("update", function(json){_self.downloaderCallback(json); });
	this.notesDownloader = new FeedDownloader("releaseNotes", function(json){_self.notesDownloaderCallback(json); });
	this.lastResponse = "";
	
	this.criticalUpdateAvailable = false;
}

UpdateChecker.prototype.setup = function() {
	var _self = this;
	document.getElementById("back-updates").onclick = function(){ _self.check(true, 1); };
	document.getElementById("back_no_update_ok").onclick = function(){ _self.closeNoUpdateWindow(); };
	
	document.getElementById("sheet-button-download-now").setAttribute("onclick", "upd.downloadUpdate();");
	document.getElementById("sheet-button-download-later").setAttribute("onclick", "upd.updateLater();");
	document.getElementById("sheet-button-skip-version").setAttribute("onclick", "upd.skipUpdate();");
	
	document.getElementById("updatebadge").setAttribute("onclick", "upd.updateModeButton();");

	setInterval(function(){ _self.check(false, 0); }, (60 * 60 * 24) * 1000);
}

UpdateChecker.prototype.downloadUpdate = function() {
	widget.openURL(update_url + "?op=get_file&number=18");
	sht.closeSheet(1);
	this.waitingForEpgSheet = false;
}

UpdateChecker.prototype.updateLater = function() {
	this.dlLaterVersion = this.newVersion;
	sht.closeSheet(1);
	this.waitingForEpgSheet = false;
}

UpdateChecker.prototype.skipUpdate = function() {
	savePref(this.newVersion, "skipVersion");
	sht.closeSheet(1);
	document.getElementById("updatebadge").style.display = "none";
	document.getElementById("updatebadgecritical").style.display = "none";
	this.waitingForEpgSheet = false;
}

UpdateChecker.prototype.closeNoUpdateWindow = function() {
	document.getElementById("back_no_update").style.display = "none";
}

UpdateChecker.prototype.notesDownloaderCallback = function(json) {
	this.releaseNotes = json;
	if(!this.releaseNotes || this.releaseNotes.length == 0)
		this.releaseNotes = Array("Release notes unavailable");
	this.processFinal();
}

UpdateChecker.prototype.downloaderCallback = function(json) {
	this.lastResponse = json;
	if(!this.lastResponse || this.lastResponse == "The requested file type does not exist." || this.lastResponse == "Version information not available." || this.lastResponse.length < 8)
		this.lastResponse = "0000000NN";
	/**
	 * The download version string looks like XXYYZZZTS
	 *
	 * Where
	 * XX  - Major Version Number
	 * YY  - Minor Version Number
	 * ZZZ - Build Number
	 * T   - Version Type ( A - Alpha, B - Beta, N - Normal )
	 * S   - Update Severity ( Y - Critical, N - Normal )
	 */

	var currentVersionString = this.lastResponse.substring(0,9);
	var currentVersion = currentVersionString.substring(0,8);
	var updateSeverity = currentVersionString.substring(8,9);

	this.newVersion = currentVersion;
	this.updateSeverity = updateSeverity;

	var currentDate = new Date();
	this.lastCheck = currentDate.getTime();
	/**
	 * The user should be presented with an update dialog if
	 * 1) There is a new version
	 * 2) The user has not hit "Download Later", updateOfferedVersion != currentVersion
	 * 3) The user has not hot "Skip Version", skipVersion != currentVersion
	 */
	 
	/**
	 * No new version
	 */

	var skip = loadPref("skipVersion");
	if(skip && skip.length > 0){
		if(skip == currentVersion){
			if(this.mode == 1)
				this.showNoUpdateWindow();
			return;
		}
	}

	if(currentVersion <= widgetVersion){
		if(this.mode == 1)
			this.showNoUpdateWindow();
		return;
	}
	if(this.updateSeverity == "C"){
		this.notesDownloader.buildURL();
		if(initialSetupComplete)
			rmtQueue.setSubActive(kUpdateCheckerFeed);
		this.notesDownloader.fetchURL();
		return;
	}
			
	/**
	 * User has hit "Download Later" and it is for the current version.
	 */ 
	if (this.dlLaterVersion && this.dlLaterVersion == currentVersion && this.mode == 0)
		return;
		
	/**
	 * User has hit "Skip" and it is for the current version.
	 */ 
	if (this.skipVersion && this.skipVersion == currentVersion)
		return;
		
		
	/*
		If we get to here then we need to display the update window.
		But first we need to get the release notes
	*/	
	this.notesDownloader.buildURL();
	if(initialSetupComplete)
		rmtQueue.setSubActive(kUpdateCheckerFeed);
	this.notesDownloader.fetchURL();
}


UpdateChecker.prototype.processFinal = function() {
	
	this.showUpdateWindow();
}

UpdateChecker.prototype.check = function(user_initiated, mode) {
	this.mode = mode;

	this.downloader.buildURL();
	if(initialSetupComplete)
		rmtQueue.setSubActive(kUpdateCheckerFeed);
	this.downloader.fetchURL();
}


UpdateChecker.prototype.showNoUpdateWindow = function(){
	document.getElementById("back_update_version_message").innerText = "IceTV " + this.versionNumberFromString(widgetVersion) + " is currently the newest version available."
		
	if(parseInt(document.getElementById("channel_list").style.height, 10) > 137)
		document.getElementById("back_no_update").style.top = (((parseInt(document.getElementById("channel_list").style.height, 10) - 137) / 2) + parseInt(document.getElementById("channel_list").style.top, 10)) + "px";
	else
		document.getElementById("back_no_update").style.top = document.getElementById("channel_list").style.top;
	document.getElementById("back_no_update").style.display = "block";
}

UpdateChecker.prototype.showUpdateWindow = function(){	
	if(this.updateSeverity == "C"){
		this.criticalUpdateAvailable = true;
		ath.disableWidget();
		ath.clearContent();

		document.getElementById("update-checker-text").innerText = "IceTV widget version '" + this.versionNumberFromString(this.newVersion) + "' is now available (you have version " + this.versionNumberFromString(widgetVersion) + "). This update is a critical update and required for all users."
		document.getElementById("critical-update-release-notes").innerHTML = this.releaseNotes;

		document.getElementById("main_container").style.height = "74px"
		document.getElementById("main_base").style.top = "146px"
		window.resizeTo(window.outerWidth, 181);

		document.getElementById("sheet-button-download-later").style.display = "none"
		document.getElementById("sheet-button-skip-version").style.display = "none"
		document.getElementById("updatebadgecritical").style.display = "block"
	} else {
		document.getElementById("update-checker-text").innerText = "IceTV widget version '" + this.versionNumberFromString(this.newVersion) + "' is now available (you have version " + this.versionNumberFromString(widgetVersion) + "). Would you like to download it now?"
		document.getElementById("critical-update-release-notes").innerHTML = this.releaseNotes;
		document.getElementById("sheet-button-download-later").style.display = "block"
		document.getElementById("sheet-button-skip-version").style.display = "block"
		document.getElementById("updatebadge").style.display = "block"
	}
	

	if(sht.epgSheetActive){
		this.waitingForEpgSheet = true;
		return;	
	}

	if(sht.updateSheetActive){
		return;	
	}

	document.getElementById("sheet-container").style.display = "block";
	document.getElementById("critical-update-sheet").style.opacity = 0;
	document.getElementById("critical-update-sheet").style.display = "block";
	var sheetHeight = parseInt(document.defaultView.getComputedStyle(document.getElementById("critical-update-sheet"),"").getPropertyValue("height"), 10);
	document.getElementById("sheet-container").style.height = sheetHeight + "px"
	document.getElementById("critical-update-sheet").style.top = -sheetHeight + "px"
	document.getElementById("critical-update-sheet").style.opacity = 1;

	var container;
	if(selectedPage == 0)
		container = document.getElementById("epg_container");
	else if(selectedPage == 1)
		container = document.getElementById("popular_container");
	else if(selectedPage == 2)
		container = document.getElementById("myguideupcoming_container");
	else if(selectedPage == 3)
		container = document.getElementById("myguideseries_container");
	else if(selectedPage == 4)
		container = document.getElementById("recommended_container");

	if(container)
		container.style.opacity = 0.4;

	this.waitingForEpgSheet = false;
	
	if(this.mode == 1){
		sht.manuallyOpenedSheet(1);
		showFront();
		setTimeout("sht.showSheet(1);", 2000);
	} else 
		sht.showSheet(1);
}

UpdateChecker.prototype.epgSheetClosed = function(){
	if(this.waitingForEpgSheet){
		this.waitingForEpgSheet = false;
		this.showUpdateWindow();
	}
}

UpdateChecker.prototype.updateModeButton = function(){
//	DEBUGLOG("updateModeButton");
	if(sht.epgSheetActive){
		this.waitingForEpgSheet = true;
		sht.closeSheet(0);
		return;
	}

	if(sht.updateSheetActive)
		return
	
	this.showUpdateWindow();
}

UpdateChecker.prototype.versionNumberFromString = function(versionString){
	/**
	 * The version string looks like XXYYZZZT
	 *
	 * Where
	 * XX  - Major Version Number
	 * YY  - Minor Version Number
	 * ZZZ - Build Number
	 * T   - Version Type ( A - Alpha, B - Beta, N - Normal )
	 */
	 
	var majorVersion = parseInt(versionString.substring(0,2), 10);
	var minorVersion = parseInt(versionString.substring(2,4), 10);
	var buildNumber = parseInt(versionString.substring(4,7), 10);
	var versionType = versionString.substring(7,8);
	var versionTypeString;
	
	if(versionType == "A")
		return majorVersion + "." + minorVersion + " Alpha " + " (" + buildNumber + ")";
	else if(versionType == "B")
		return majorVersion + "." + minorVersion + " Beta " + " (" + buildNumber + ")";
	else 
		return majorVersion + "." + minorVersion + " (" + buildNumber + ")";
}